//---------------------------------------------------------------------------
#include <vcl.h>
#include <jpeg.hpp>
#include <shellapi.h>
#pragma hdrstop

#include "miscCameraControl.h"
#include "miscVideoControl.h"
#include "miscVideoQuality.h"
#include "MainForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "VidGrab"
#pragma resource "*.dfm"

TfrmMainForm *frmMainForm;

//---------------------------------------------------------------------------
__fastcall TfrmMainForm::TfrmMainForm(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall AssignListToComboBox (TComboBox* ComboBox, AnsiString List, int Index)
{
   ComboBox->Items->Text = List;
   if (ComboBox->Items->Count > 0) {
      ComboBox->ItemIndex = Index;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::FormCreate(TObject *Sender)
{
   iFreeHandMouseDrawing = 0;                // used for free-hand mouse drawing
   FreeHandMouseDrawingEnabled = false;      // used for free-hand mouse drawing
   SearchBmpFilesOpened = false;             // used for the "video from bitmaps" in the VideoGrabberVideoFromBitmaps_NextFrameNeeded event

   StartupWidth = Width;
   StartupHeight = Height;

   // - controls using global TVideoGrabber properties, e.g. like the VideoDevices list, are updated from the FormCreate event
   // - controls that depend of the current video device are set in the OnVideoDeviceSelected event
   // - controls that depend of the current audio device are set in the OnAudioDeviceSelected event


   ///////////////////////// Controls on main form
   edtStoragePath->Text = VideoGrabber->StoragePath;

   ///////////////////////// Video source tab
   AssignListToComboBox (cboVideoSource, VideoGrabber->VideoSources, VideoGrabber->VideoSource);
   btnAutoRefreshPreview->Checked = VideoGrabber->AutoRefreshPreview;
   cboVideoSource->ItemIndex = VideoGrabber->VideoSource;
   chkScreenRecordingWithCursor->Checked = VideoGrabber->ScreenRecordingWithCursor;
   chkMultiplexedInputEmulation->Checked = VideoGrabber->MultiplexedInputEmulation;


   ///////////////////////// Audio device tab
   chkRenderAudioDevice->Checked = VideoGrabber->AudioDeviceRendering;
   chkMuteAudioRendering->Checked = VideoGrabber->MuteAudioRendering;

   ///////////////////////// RECORDING tab
   rdgRecordingMethod->ItemIndex = VideoGrabber->RecordingMethod;
   edtPreallocSize->Text = IntToStr (VideoGrabber->PreallocCapFileSizeInMB);
   chkRecordingCanPause->Checked = VideoGrabber->RecordingCanPause;
   chkPreallocFile->Checked = VideoGrabber->PreallocCapFileEnabled;
   chkOpenDML->Checked = VideoGrabber->AVIFormatOpenDML;
   chkPreserveNativeFormat->Checked = VideoGrabber->RecordingInNativeFormat;

   rdgCompressType->ItemIndex = VideoGrabber->CompressionType;
   rdgCompressMode->ItemIndex = VideoGrabber->CompressionMode;
   chkAudioRecording->Checked = VideoGrabber->AudioRecording;

   AssignListToComboBox (cboVideoCompressors, VideoCompressors(), VideoGrabber->VideoCompressor);
   AssignListToComboBox (cboAudioCompressors, AudioCompressors(), VideoGrabber->AudioCompressor);

   ///////////////////////// network Streaming tab

   rdgNetworkStreaming->ItemIndex = VideoGrabber->NetworkStreaming;
   edtPublishingPoint->Text = VideoGrabber->ASFMediaServerPublishingPoint;
   edtNetworkPort->Text = IntToStr (VideoGrabber->ASFNetworkPort);
   edtNetworkMaxUsers->Text = IntToStr (VideoGrabber->ASFNetworkMaxUsers);

   edtASFVideoWidth->Text = IntToStr (VideoGrabber->ASFVideoWidth);
   edtASFVideoHeight->Text = IntToStr (VideoGrabber->ASFVideoHeight);
   edtASFVideoBitRate->Text = IntToStr (VideoGrabber->ASFVideoBitRate);
   edtASFVideoMaxKeyFrameSpacing->Text = IntToStr (VideoGrabber->ASFVideoMaxKeyFrameSpacing);
   edtASFVideoQuality->Text = IntToStr (VideoGrabber->ASFVideoQuality);
   edtASFAudioChannels->Text = IntToStr (VideoGrabber->ASFAudioChannels);
   chkASFFixedFrameRate->Checked = VideoGrabber->ASFFixedFrameRate;
   cmbASFDeinterlaceMode->ItemIndex = VideoGrabber->ASFDeinterlaceMode;

   edtProfileIndex->Text = IntToStr (VideoGrabber->ASFProfile);

   ///////////////////////// Player tab
   updPlayerSpeedRatio->Position = VideoGrabber->PlayerSpeedRatio * 1000;
   edtPlayerSpeedRatio->Text = FormatFloat ("0.0", VideoGrabber->PlayerSpeedRatio);

   updPlayerFastSeekSpeed->Position = (short) VideoGrabber->PlayerFastSeekSpeedRatio;
   edtPlayerFastSeekSpeed->Text = IntToStr (VideoGrabber->PlayerFastSeekSpeedRatio);

   btnAutoPlay->Checked = VideoGrabber->AutoStartPlayer;
   chkPlayerAudioRendering->Checked = VideoGrabber->PlayerAudioRendering;
   rdgDVPlaybackSize->ItemIndex = VideoGrabber->PlayerDVSize;
   chkRefreshPausedDisplay->Checked = VideoGrabber->PlayerRefreshPausedDisplay;

   ///////////////////////// Frame grabber tab
   edtAVIAutoFilePrefix->Text = VideoGrabber->AutoFilePrefix;
   rgdFrameGrabberFormat->ItemIndex = VideoGrabber->FrameGrabberRGBFormat;
   rgdFrameGrabber->ItemIndex = VideoGrabber->FrameGrabber;
   rdgAutoFileName->ItemIndex = VideoGrabber->AutoFileName;

   ///////////////////////// Frame capture tab
   edtBurstCount->Text = IntToStr (VideoGrabber->BurstCount);
   edtBurstInterval->Text = IntToStr (VideoGrabber->BurstInterval);
   tbrCaptureZoomSize->Position = VideoGrabber->FrameCaptureZoomSize;

   ///////////////////////// Motion detection tab

   chkMotionDetectionEnabled->Checked = VideoGrabber->MotionDetector_Enabled;
   chkCompareRed->Checked = VideoGrabber->MotionDetector_CompareRed;
   chkCompareGreen->Checked = VideoGrabber->MotionDetector_CompareGreen;
   chkCompareBlue->Checked = VideoGrabber->MotionDetector_CompareBlue;
   chk_GreyScale->Checked = VideoGrabber->MotionDetector_GreyScale;
   chkReduceVideoNoise->Checked = VideoGrabber->MotionDetector_ReduceVideoNoise;
   edtGrid->Text = VideoGrabber->MotionDetector_Grid;
   mmoSensitivityGrid->Text = VideoGrabber->MotionDetector_Get2DTextGrid();
   edtGridColCount->Text = IntToStr (VideoGrabber->MotionDetector_GridXCount);
   edtGridRowCount->Text = IntToStr (VideoGrabber->MotionDetector_GridYCount);
   chkTriggered->Checked = VideoGrabber->MotionDetector_Triggered;

   ///////////////////////// TV Tuner tab
   edtCountryCode->Text = IntToStr (VideoGrabber->TVCountryCode);
   rdgTunerInputType->ItemIndex = VideoGrabber->TVTunerInputType;
   edtTVChannel->Text = IntToStr (VideoGrabber->TVChannel);
   chkEnableFrequencyOverrides->Checked = VideoGrabber->TVUseFrequencyOverrides;

   ///////////////////////// Overlay tab
   mmoTextOverlayString->Text = VideoGrabber->TextOverlay_String;
   chkTextOverlayEnabled->Checked = VideoGrabber->TextOverlay_Enabled;
   chkTextOverlayTransparent->Checked = VideoGrabber->TextOverlay_Transparent;
   cmbBitmapOverlayTransparentColor->ItemIndex = 0;
   cmbBitmapOverlayTransparentColorItemIndex = 0; // intermediary variable used to read the ItemIndex from the OnFrameOverlayUsingDC event
   updBitmapOverlayAlphaBendPosition = 100;        // intermediary variable used to read the Position from the OnFrameOverlayUsingDC event

   ///////////////////////// Video processing tab
   rdgVideoRotation->ItemIndex = VideoGrabber->VideoProcessing_Rotation;
   tbrBrightness->Position = - VideoGrabber->VideoProcessing_Brightness;
   tbrConstrast->Position = - VideoGrabber->VideoProcessing_Contrast;
   tbrSaturation->Position = - VideoGrabber->VideoProcessing_Saturation;
   tbrHue->Position = VideoGrabber->VideoProcessing_Hue;

   rdgDeinterlacing->ItemIndex = VideoGrabber->VideoProcessing_Deinterlacing;
   chkGreyScale->Checked = VideoGrabber->VideoProcessing_GrayScale;
   chkInvertColors->Checked = VideoGrabber->VideoProcessing_InvertColors;
   chkFlipVertical->Checked = VideoGrabber->VideoProcessing_FlipVertical;
   chkFlipHorizontal->Checked = VideoGrabber->VideoProcessing_FlipHorizontal;

   chkCroppingEnabled->Checked = VideoGrabber->Cropping_Enabled;
   chkCroppingOutbounds->Checked = VideoGrabber->Cropping_Outbounds;
   edtCroppingWidth->Text = IntToStr (VideoGrabber->Cropping_Width);
   edtCroppingHeight->Text = IntToStr (VideoGrabber->Cropping_Height);
   trkCroppingZoom->Position = VideoGrabber->Cropping_Zoom * 100;

   // trkCroppingX.Max and trkCroppingY.Max are will be set from the OnResizeVideo event
   trkCroppingY->Position = VideoGrabber->Cropping_Y;
   trkCroppingX->Position = VideoGrabber->Cropping_X;


   ///////////////////////// Display tab
   chkBorder->Checked = VideoGrabber->BorderStyle == bsSingle;
   chkTransparency->Checked = VideoGrabber->ColorKeyEnabled;
   rdgVideoRenderer->ItemIndex = VideoGrabber->VideoRenderer;
   RefreshDisplayOptions();

   ///////////////////////// reencoding tab
   chkWMVOutput->Checked = VideoGrabber->Reencoding_WMVOutput;
   chkIncludeVideoStream->Checked = VideoGrabber->Reencoding_IncludeVideoStream;
   chkIncludeAudioStream->Checked = VideoGrabber->Reencoding_IncludeAudioStream ;
   chkUseFrameGrabber->Checked = VideoGrabber->Reencoding_UseFrameGrabber;
   chkUseCurrentVideoCompressor->Checked = VideoGrabber->Reencoding_UseVideoCompressor;
   chkUseCurrentAudioCompressor->Checked = VideoGrabber->Reencoding_UseAudioCompressor;
   edtStartTime->Text = IntToStr (VideoGrabber->Reencoding_StartTime);
   edtStopTime->Text = IntToStr (VideoGrabber->Reencoding_StopTime);
   rdgReencodingMethod->ItemIndex = Integer (VideoGrabber->Reencoding_Method);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::FormShow(TObject *Sender)
{
   RefreshVideoDeviceControls();
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//                            VIDEO DEVICE TAB
////////////////////////////////////////////////////////////////////////////////

void __fastcall TfrmMainForm::VideoGrabberVideoDeviceSelected(
      TObject *Sender)
{
   // from this event update the properties that depend of the current video capture device->
   // This event occurs when you assign a new value to the VideoDevice property->

   AssignListToComboBox (cboVideoSizes, VideoGrabber->VideoSizes, VideoGrabber->VideoSize);
   AssignListToComboBox (cboVideoSubtypes, VideoGrabber->VideoSubtypes, VideoGrabber->VideoSubtype);
   AssignListToComboBox (cboAnalogVideoStandard, VideoGrabber->AnalogVideoStandards, VideoGrabber->AnalogVideoStandard);
   AssignListToComboBox (cboVideoInputs, VideoGrabber->VideoInputs, VideoGrabber->VideoInput);

   RefreshVideoDeviceControls();
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cboVideoSourceChange(TObject *Sender)
{
  switch (cboVideoSource->ItemIndex) {
     case 0: VideoGrabber->VideoSource = vs_VideoCaptureDevice; break;
     case 1: VideoGrabber->VideoSource = vs_ScreenRecording;  ; break;
     case 2: VideoGrabber->VideoSource = vs_VideoFileOrURL;   ; break;
     case 3: VideoGrabber->VideoSource = vs_JPEGsOrBitmaps; ; break;
  }
}

//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::cboVideoDevicesChange(TObject *Sender)
{
   VideoGrabber->VideoDevice = cboVideoDevices->ItemIndex;
   // when selecting another video device, the OnVideoDeviceSelected event occurs.
   // be sure to refresh all your device-dependent controls from this event.
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnStartPreviewClick(TObject *Sender)
{
   VideoGrabber->StartPreview();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnStopPreviewClick(TObject *Sender)
{
   VideoGrabber->StopPreview();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnPausePreviewClick(TObject *Sender)
{
   VideoGrabber->PausePreview();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnResumePreviewClick(TObject *Sender)
{
   VideoGrabber->ResumePreview();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnAutoRefreshPreviewClick(TObject *Sender)
{
   VideoGrabber->AutoRefreshPreview = btnAutoRefreshPreview->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cboVideoInputsChange(TObject *Sender)
{
   VideoGrabber->VideoInput = cboVideoInputs->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cboVideoSizesChange(TObject *Sender)
{
   VideoGrabber->VideoSize = cboVideoSizes->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cboAnalogVideoStandardChange(TObject *Sender)
{
   VideoGrabber->AnalogVideoStandard = cboAnalogVideoStandard->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cboVideoSubtypesChange(TObject *Sender)
{
   VideoGrabber->VideoSubtype = cboVideoSubtypes->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkSetLogoClick(TObject *Sender)
{
   if (chkSetLogo->Checked) {
      imgLogo->Visible = false;
      VideoGrabber->SetLogoFromTImage (imgLogo);
      VideoGrabber->LogoLayout = lg_Stretched;
      VideoGrabber->LogoDisplayed = true;
   }
   else {
      VideoGrabber->LogoDisplayed = false;
   }
}
//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::btnDeviceDialogClick(TObject *Sender)
{
   VideoGrabber->ShowDialog (dlg_VideoDevice);
   RefreshVideoDeviceControls();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnStreamDialogClick(TObject *Sender)
{
   // WARNING: USUALLY, THIS DIALOG ALLOWS TO CHANGE ONLY 1 PARAMETER AT THE SAME TIME
   VideoGrabber->ShowDialog (dlg_StreamConfig);
   VideoGrabberVideoDeviceSelected (this); // to refresh the controls, if (required
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnCameraControlDialogClick(TObject *Sender)
{
   ShowFormAboveVideo (frmCameraControl);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnVideoQualityDialogClick(TObject *Sender)
{
   ShowFormAboveVideo (frmVideoQuality);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnVideoControlDialogClick(TObject *Sender)
{
   ShowFormAboveVideo (frmVideoControl);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rdgPreferredVideoSizeClick(TObject *Sender)
{
   switch (rdgPreferredVideoSize->ItemIndex) {
      case 0: VideoGrabber->UseNearestVideoSize (0, 0, chkStretchPreferredVideoSize->Checked); break; // disables the nearest size feature, uses the device"s VideoSize index in the VideoSizes list
      case 1: VideoGrabber->UseNearestVideoSize ( 88,  60, chkStretchPreferredVideoSize->Checked); break;
      case 2: VideoGrabber->UseNearestVideoSize ( 88,  72, chkStretchPreferredVideoSize->Checked); break;
      case 3: VideoGrabber->UseNearestVideoSize (180, 120, chkStretchPreferredVideoSize->Checked); break;
      case 4: VideoGrabber->UseNearestVideoSize (180, 144, chkStretchPreferredVideoSize->Checked); break;
      case 5: VideoGrabber->UseNearestVideoSize (320, 240, chkStretchPreferredVideoSize->Checked); break;
      case 6: VideoGrabber->UseNearestVideoSize (352, 288, chkStretchPreferredVideoSize->Checked); break;
      case 7: VideoGrabber->UseNearestVideoSize (360, 240, chkStretchPreferredVideoSize->Checked); break;
      case 8: VideoGrabber->UseNearestVideoSize (360, 288, chkStretchPreferredVideoSize->Checked); break;
      case 9: VideoGrabber->UseNearestVideoSize (640, 480, chkStretchPreferredVideoSize->Checked); break;
      case 10: VideoGrabber->UseNearestVideoSize (720, 480, chkStretchPreferredVideoSize->Checked); break;
      case 11: VideoGrabber->UseNearestVideoSize (720, 576, chkStretchPreferredVideoSize->Checked); break;
      case 12: VideoGrabber->UseNearestVideoSize (768, 576, chkStretchPreferredVideoSize->Checked); break;
      // you can add your own preferred formats
   }
   cboVideoSizes->Enabled = rdgPreferredVideoSize->ItemIndex == 0;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnFrameRateClick(TObject *Sender)
{
   float Value;

   Value = StrToFloatDef (edtFrameRate->Text, -1);
   VideoGrabber->FrameRate = Value;
   if (VideoGrabber->CurrentState == cs_Preview) {
      mmoLog->Lines->Add ("current frame rate: " + FormatFloat ("0.00", VideoGrabber->CurrentFrameRate) + " fps.");
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkMultiplexedAutoSwitchClick(
      TObject *Sender)
{
   if (chkMultiplexedAutoSwitch->Checked) {
      VideoGrabber->MultiplexedRole = mr_MultiplexedMosaic4;
   }
   else {
      VideoGrabber->MultiplexedRole = mr_NotMultiplexed;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkMultiplexedInputEmulationClick(
      TObject *Sender)
{
   VideoGrabber->MultiplexedInputEmulation = chkMultiplexedInputEmulation->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkReduceDVFrameRateClick(TObject *Sender)
{
   VideoGrabber->DVReduceFrameRate = chkReduceDVFrameRate->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnOpenFileOrUrlClick(TObject *Sender)
{
   dlgOpen->FileName = "*.mpg;*.mpeg;*.avi;*.wav;*.asf;*.wmv;*.vob";
   if (dlgOpen->Execute()) {
      edtVideoSourceFileOrUrl->Text = dlgOpen->FileName;
   }

}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtVideoSourceFileOrUrlChange(
      TObject *Sender)
{
   VideoGrabber->VideoSource_FileOrURL = edtVideoSourceFileOrUrl->Text;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkScreenRecordingWithCursorClick(
      TObject *Sender)
{
   VideoGrabber->ScreenRecordingWithCursor = chkScreenRecordingWithCursor->Checked;
}

//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::VideoGrabberPreviewStarted(TObject *Sender)
{
   mmoLog->Lines->Add ("");
   mmoLog->Lines->Add ("video size: " + IntToStr (VideoGrabber->VideoWidth) + "x" + IntToStr (VideoGrabber->VideoHeight));
   mmoLog->Lines->Add ("display size: " + IntToStr (VideoGrabber->Display_VideoWidth) + "x" + IntToStr (VideoGrabber->Display_VideoHeight));
   mmoLog->Lines->Add ("current frame rate: " + FormatFloat ("0.00", VideoGrabber->CurrentFrameRate) + " fps.");
   mmoLog->Lines->Add ("");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberFrameProgress(TObject *Sender, DWORD FrameNumber, Int64 FrameTime, int FrameId)
{
   AnsiString DVTimeCode;
   AnsiString DVDateTime;

   int SampleTime_Hour = VideoGrabber->GetFrameInfo (FrameId, fi_SampleTime_Hour);
   int SampleTime_Min  = VideoGrabber->GetFrameInfo (FrameId, fi_SampleTime_Min);
   int SampleTime_Sec  = VideoGrabber->GetFrameInfo (FrameId, fi_SampleTime_Sec);
   int SampleTime_Hs   = VideoGrabber->GetFrameInfo (FrameId, fi_SampleTime_Hs);

   TVarRec SampleTimeVars[] = {SampleTime_Hour, SampleTime_Min, SampleTime_Sec, SampleTime_Hs};

   edtFrameCount->Text = "frame: " + IntToStr (FrameNumber)
               + " dropped:" + IntToStr (VideoGrabber->GetFrameInfo (FrameId, fi_DroppedFrameCount))
               + " time:" + Format ("%.2d:%.2d:%.2d %.2d", SampleTimeVars, (ARRAYSIZE(SampleTimeVars) -1));

   if (VideoGrabber->GetFrameInfo (FrameId, fi_DVTimeCode_IsAvailable)) {
      int DVTimeCode_Hour = VideoGrabber->GetFrameInfo (FrameId, fi_DVTimeCode_Hour);
      int DVTimeCode_Min = VideoGrabber->GetFrameInfo (FrameId, fi_DVTimeCode_Min);
      int DVTimeCode_Sec = VideoGrabber->GetFrameInfo (FrameId, fi_DVTimeCode_Sec);
      int DVTimeCode_Ff = VideoGrabber->GetFrameInfo (FrameId, fi_DVTimeCode_Ff);
      TVarRec TimeCodeVars[] = {DVTimeCode_Hour, DVTimeCode_Min, DVTimeCode_Sec, DVTimeCode_Ff};
      DVTimeCode = Format ("tc: %.2d:%.2d:%.2d %.2d", TimeCodeVars, (ARRAYSIZE(TimeCodeVars) -1));
   }
   else {
       DVTimeCode = "";
   }

   if (VideoGrabber->GetFrameInfo (FrameId, fi_DVDateTime_IsAvailable)) {
      int DVDateTime_Year = VideoGrabber->GetFrameInfo (FrameId, fi_DVDateTime_Year);
      int DVDateTime_Month = VideoGrabber->GetFrameInfo (FrameId, fi_DVDateTime_Month);
      int DVDateTime_Day = VideoGrabber->GetFrameInfo (FrameId, fi_DVDateTime_Day);
      int DVDateTime_Hour = VideoGrabber->GetFrameInfo (FrameId, fi_DVDateTime_Hour);
      int DVDateTime_Min = VideoGrabber->GetFrameInfo (FrameId, fi_DVDateTime_Min);
      int DVDateTime_Sec = VideoGrabber->GetFrameInfo (FrameId, fi_DVDateTime_Sec);
      TVarRec DVDateTimeVars[] = {DVDateTime_Day, DVDateTime_Month, DVDateTime_Year, DVDateTime_Hour, DVDateTime_Min, DVDateTime_Sec};
       DVDateTime = Format ("dv date: %.2d/%.2d/%.2d %.2d:%.2d:%.2d", DVDateTimeVars, (ARRAYSIZE(DVDateTimeVars) -1));
   }
   else {
       DVDateTime = "";
   }

   edtDVDateTime->Text = DVTimeCode + " " + DVDateTime;
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//                           AUDIO TAB
////////////////////////////////////////////////////////////////////////////////

void __fastcall TfrmMainForm::VideoGrabberAudioDeviceSelected(
      TObject *Sender)
{
   // from this event set the parameters that change when the current audio capture device change

   cboAudioDevices->ItemIndex = VideoGrabber->AudioDevice; // be sure the listbox index matches the current index value

   AssignListToComboBox (cboAudioFormats, VideoGrabber->AudioFormats, VideoGrabber->AudioFormat);
   AssignListToComboBox (cboAudioInputs, VideoGrabber->AudioInputs, VideoGrabber->AudioInput);
   tbrAudioInputLevel->Position = VideoGrabber->AudioInputLevel;
   tbrAudioInputBalance->Position = VideoGrabber->AudioInputBalance;
   chkAudioInputMono->Checked = VideoGrabber->AudioInputMono;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkRenderAudioDeviceClick(TObject *Sender)
{
   VideoGrabber->AudioDeviceRendering = chkRenderAudioDevice->Checked;

}
//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::cboAudioDevicesChange(TObject *Sender)
{
   VideoGrabber->AudioDevice = cboAudioDevices->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cboAudioInputsChange(TObject *Sender)
{
   VideoGrabber->AudioInput = cboAudioInputs->ItemIndex;
}
//---------------------------------------------------------------------------


void __fastcall TfrmMainForm::tbrAudioInputLevelChange(TObject *Sender)
{
   VideoGrabber->AudioInputLevel = tbrAudioInputLevel->Position;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrAudioInputBalanceChange(TObject *Sender)
{
   VideoGrabber->AudioInputBalance = tbrAudioInputBalance->Position;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkAudioInputMonoClick(TObject *Sender)
{
   VideoGrabber->AudioInputMono = chkAudioInputMono->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrAudioVolumeChange(TObject *Sender)
{
   VideoGrabber->AudioVolume = tbrAudioVolume->Position;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrAudioBalanceChange(TObject *Sender)
{
   VideoGrabber->AudioBalance = tbrAudioBalance->Position;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkMuteAudioRenderingClick(TObject *Sender)
{
   VideoGrabber->MuteAudioRendering = chkMuteAudioRendering->Checked;
}
//---------------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////////////
//                             RECORDING TAB
////////////////////////////////////////////////////////////////////////////////

void __fastcall TfrmMainForm::btnStartRecordingImmediatelyClick(
      TObject *Sender)
{
   if (chkAVISaveToThisFile->Checked) {
      VideoGrabber->RecordingFileName = edtRecordingFileName->Text;
   }
   else {
      VideoGrabber->RecordingFileName = "";
   }
   VideoGrabber->HoldRecording = false; // let's start recording as soon as the graph is built.
   VideoGrabber->StartRecording();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnStartRecordingControlledClick(
      TObject *Sender)
{
   if (chkAVISaveToThisFile->Checked) {
      VideoGrabber->RecordingFileName = edtRecordingFileName->Text;
   }
   else {
      VideoGrabber->RecordingFileName = "";
   }
   VideoGrabber->HoldRecording = True;
   VideoGrabber->StartRecording();

   // Enabling HoldRecording prevents the recording to start when the recording graph is built.
   // after the OnRecordingReadyToStart event occurs, you can accurately start the
   // recording by invoking ResumeRecording.
}
//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::btnStopRecordingClick(TObject *Sender)
{
   VideoGrabber->StopRecording();
}

//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnRenewRecordingFileClick(TObject *Sender)
{
   if (chkAVISaveToThisFile->Checked) {
      VideoGrabber->RecordingFileName = edtRecordingFileName->Text;
   }
   else {
      // keyword that indicates TVideoGrabber to generate a new recording file name automatically
      VideoGrabber->RecordingFileName = "RENEW";
   }
}

//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::rdgRecordingMethodClick(TObject *Sender)
{
   VideoGrabber->RecordingMethod = TRecordingMethod (rdgRecordingMethod->ItemIndex);
   RefreshVideoDeviceControls();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkRecordingCanPauseClick(TObject *Sender)
{
   VideoGrabber->RecordingCanPause = chkRecordingCanPause->Checked;
   RefreshVideoDeviceControls();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnPauseRecordingClick(TObject *Sender)
{
   VideoGrabber->PauseRecording();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnResumeRecordingClick(TObject *Sender)
{
   VideoGrabber->ResumeRecording();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkOpenDMLClick(TObject *Sender)
{
   VideoGrabber->AVIFormatOpenDML = chkOpenDML->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cboAudioFormatsChange(TObject *Sender)
{
   VideoGrabber->AudioFormat = TAudioFormat (cboAudioFormats->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkAudioRecordingClick(TObject *Sender)
{
   VideoGrabber->AudioRecording = chkAudioRecording->Checked;
   RefreshVideoDeviceControls();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkPreserveNativeFormatClick(TObject *Sender)
{
   VideoGrabber->RecordingInNativeFormat = chkPreserveNativeFormat->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnNativeFormatClick(TObject *Sender)
{
   AnsiString Msg;

   Msg += "Means that the recording stream will not be uncompressed.\n";
   Msg += "\n";
   Msg += "MUST BE ENABLED:\n";
   Msg += "- to record hardware-encoded MPEG\n";
   Msg += "- to record or DV devices in native DV format\n";
   Msg += "- to record the DV date/time into the video clip and retrieve it during playback\n";
   Msg += "\n";
   Msg += "MUST BE DISABLED:\n";
   Msg += "- to record the text and graphics overlays into the video clip, and retrieve them during playback\n";
   ShowMessage (Msg);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::chkPreallocFileClick(TObject *Sender)
{
   VideoGrabber->PreallocCapFileEnabled = chkPreallocFile->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnCreatePreallocFileNowClick(
      TObject *Sender)
{
   VideoGrabber->CreatePreallocCapFile();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtPreallocSizeChange(TObject *Sender)
{
   VideoGrabber->PreallocCapFileSizeInMB = StrToIntDef (edtPreallocSize->Text, 100);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cboVideoCompressorsChange(TObject *Sender)
{
   VideoGrabber->VideoCompressor = cboVideoCompressors->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cboAudioCompressorsChange(TObject *Sender)
{
   VideoGrabber->AudioCompressor = cboAudioCompressors->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnVideoCompressorSettingsClick(
      TObject *Sender)
{
   VideoGrabber->ShowDialog (dlg_VideoCompressor);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnAudioCompressorSettingsClick(
      TObject *Sender)
{
   VideoGrabber->ShowDialog (dlg_AudioCompressor);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rdgCompressModeClick(TObject *Sender)
{
   VideoGrabber->CompressionMode = TCompressionMode (rdgCompressMode->ItemIndex);
   RefreshVideoDeviceControls();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rdgCompressTypeClick(TObject *Sender)
{
   VideoGrabber->CompressionType = TCompressionType (rdgCompressType->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberRecordingReadyToStart(
      TObject *Sender)
{
   if (MessageDlg ("Ready to start recording!", mtInformation, TMsgDlgButtons() << mbOK << mbCancel, 0) == mrOk) {
      VideoGrabber->ResumeRecording();
   }
   else {
      VideoGrabber->StopRecording();
   }
}

void __fastcall TfrmMainForm::VideoGrabberRecordingStarted(TObject *Sender,
      AnsiString FileName)
{
   mmoLog->Lines->Add ("recording to " + FileName + ":");
   mmoLog->Lines->Add (IntToStr (VideoGrabber->RecordingWidth) + "x" + IntToStr (VideoGrabber->RecordingHeight));
   mmoLog->Lines->Add ("current frame rate: " + FormatFloat ("0.0", VideoGrabber->CurrentFrameRate) + " fps.");
   btnRenewRecordingFile->Enabled = true;
}

//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::VideoGrabberRecordingCompleted(
      TObject *Sender, AnsiString FileName, bool Success)
{
   if (Success) {
      mmoLog->Lines->Add (FileName + " successfully created.");
   }
   else {
      mmoLog->Lines->Add ("recording failed.");
   }
   btnRenewRecordingFile->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberReencodingStarted(
      TObject *Sender, AnsiString SourceFile, AnsiString DestFile)
{
   mmoLog->Lines->Add ("reencoding " + SourceFile + " to " + DestFile + "...");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberReencodingProgress(
      TObject *Sender, int Percent, Int64 Position, Int64 Duration)
{
   mmoLog->Lines->Add (IntToStr (Percent) + "% completed...");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberReencodingCompleted(
      TObject *Sender, AnsiString SourceFile, AnsiString DestFile,
      bool Success)
{
   if (Success) {
      mmoLog->Lines->Add (SourceFile + "successfully reencoded to " + DestFile);
   }
   else {
      mmoLog->Lines->Add ("ERROR: failed to reencode " + SourceFile + " to " + DestFile);
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberCreatePreallocFileStarted(
      TObject *Sender, AnsiString FileName)
{
   mmoLog->Lines->Add ("preallocating file " + FileName + "...");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberCreatePreallocFileProgress(
      TObject *Sender, int Percent, Int64 Position, Int64 Duration)
{
   mmoLog->Lines->Add (IntToStr (Percent) + "% of preallocated file completed");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberCreatePreallocFileCompleted(
      TObject *Sender, AnsiString FileName, bool Success)
{
   if (Success) {
      mmoLog->Lines->Add (FileName + "successfully preallocated.");
   }
   else {
      mmoLog->Lines->Add ("Failed to allocate " + FileName);
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberCopyPreallocDataStarted(
      TObject *Sender, AnsiString SourceFile, AnsiString DestFile)
{
   mmoLog->Lines->Add ("copying data from " + SourceFile + " to " + DestFile + "...");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberCopyPreallocDataProgress(
      TObject *Sender, int Percent, Int64 Position, Int64 Duration)
{
   mmoLog->Lines->Add ("copying data: " + IntToStr (Percent) + "% completed.");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberCopyPreallocDataCompleted(
      TObject *Sender, AnsiString SourceFile, AnsiString DestFile,
      bool Success)
{
   if (Success) {
      mmoLog->Lines->Add (SourceFile + " successfully copied to " + DestFile);
   }
   else {
      mmoLog->Lines->Add ("ERROR: failed to copy " + SourceFile + " to " + DestFile);
   }

}
//---------------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////////////
//                             FRAME GRABBER TAB
////////////////////////////////////////////////////////////////////////////////


void __fastcall TfrmMainForm::rgdFrameGrabberClick(TObject *Sender)
{
   VideoGrabber->FrameGrabber = TFrameGrabber (rgdFrameGrabber->ItemIndex);
   RefreshVideoDeviceControls();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rgdFrameGrabberFormatClick(TObject *Sender)
{
   VideoGrabber->FrameGrabberRGBFormat = TFrameGrabberRGBFormat (rgdFrameGrabberFormat->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rdgAutoFileNameClick(TObject *Sender)
{
   VideoGrabber->AutoFileName = TAutoFileName (rdgAutoFileName->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtAVIAutoFilePrefixChange(TObject *Sender)
{
   VideoGrabber->AutoFilePrefix = edtAVIAutoFilePrefix->Text;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnOneShotTBitmapClick(TObject *Sender)
{
   VideoGrabber->CaptureFrameTo (fc_TBitmap, "");
   // note 1: passing an empty file name let TVideoGrabber generate it automatically in the StoragePath directory
   // note 2: using CaptureFrameSyncTo instead of CaptureFrameTo waits for frame capture completion and returns true upon success
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnOneShotBMPFileClick(TObject *Sender)
{
   if (chkUseThisFileName->Checked) {
      VideoGrabber->CaptureFrameTo (fc_BmpFile, edtUseThisFileName->Text);
   }
   else {
      VideoGrabber->CaptureFrameTo (fc_BmpFile, "");
   }
   // note 1: passing an empty file name let TVideoGrabber generate it automatically in the StoragePath directory
   // note 2: using CaptureFrameSyncTo instead of CaptureFrameTo waits for frame capture completion and returns true upon success
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnOneShotJPEGFileClick(TObject *Sender)
{
   if (chkUseThisFileName->Checked) {
      VideoGrabber->CaptureFrameTo (fc_JpegFile, edtUseThisFileName->Text);
   }
   else {
      VideoGrabber->CaptureFrameTo (fc_JpegFile, "");
   }
   // note 1: passing an empty file name let TVideoGrabber generate it automatically in the StoragePath directory
   // note 2: using CaptureFrameSyncTo instead of CaptureFrameTo waits for frame capture completion and returns true upon success
}
//---------------------------------------------------------------------------


void __fastcall TfrmMainForm::chkUseThisFileNameClick(TObject *Sender)
{
   edtUseThisFileName->Enabled = chkUseThisFileName->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnBurstModeTBitmapClick(TObject *Sender)
{
   VideoGrabber->BurstType = fc_TBitmap;
   VideoGrabber->BurstMode = True;
   // The frame capture starts when BurstMode is set to True.
   // if BurstCount = 0 the frame capture does not stop.
   // Captured frames will be returned by the OnFrameCaptureCompleted event.
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnBurstModeBMPFileClick(TObject *Sender)
{
   VideoGrabber->BurstType = fc_BmpFile;
   VideoGrabber->BurstMode = True;
   // The frame capture starts when BurstMode is set to True.
   // if BurstCount = 0 the frame capture does not stop.
   // BMP file names will be generated automatically according to AutoFileName
   // and AutoFilePrefix. They will be returned by the OnFrameCaptureCompleted event.
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnBurstModeJPEGFileClick(TObject *Sender)
{
   VideoGrabber->BurstType = fc_JpegFile;
   VideoGrabber->BurstMode = True;
   // The frame capture starts when BurstMode is set to True.
   // if BurstCount = 0 the frame capture does not stop.
   // JPEG file names will be generated automatically according to AutoFileName
   // and AutoFilePrefix. They will be returned by the OnFrameCaptureCompleted event->
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnBurstModeStopClick(TObject *Sender)
{
   VideoGrabber->BurstMode = false;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtBurstCountChange(TObject *Sender)
{
   VideoGrabber->BurstCount = StrToIntDef (edtBurstCount->Text, 1);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtBurstIntervalChange(TObject *Sender)
{
   VideoGrabber->BurstInterval = StrToIntDef (edtBurstInterval->Text, 1);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrCaptureZoomSizeChange(TObject *Sender)
{
   VideoGrabber->FrameCaptureZoomSize = tbrCaptureZoomSize->Position;
   edtCaptureZoomSize->Text = IntToStr (tbrCaptureZoomSize->Position) + "%";
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnResetCaptureZoomSizeClick(TObject *Sender)
{
   tbrCaptureZoomSize->Position = 100;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberFrameCaptureCompleted(
      TObject *Sender, Pointer FrameBitmap, int BitmapWidth,
      int BitmapHeight, DWORD FrameNumber, Int64 FrameTime,
      TFrameCaptureDest DestType, AnsiString FileName, bool Success,
      int FrameId)
{
   TForm *DestForm;
   TImage *DestImage;
   Graphics::TBitmap *Frame_Bitmap;

   Frame_Bitmap = (Graphics::TBitmap*) FrameBitmap;

   if (Success) {
      mmoFrameCaptureLog->Lines->Add ("frame #" + IntToStr (FrameNumber) + " captured");
   }
   switch (DestType) {
      // ______ bmp or jpeg file
      case fc_BmpFile, fc_JpegFile: {
         if (Success) {
             mmoFrameCaptureLog->Lines->Add (FileName);
          }
          else {
             mmoFrameCaptureLog->Lines->Add ("error: failed to create " + FileName);
          }
       } break;

      // ______ memory bitmap
      case fc_TBitmap: {
         DestForm = NULL;
         if ((! VideoGrabber->BurstMode) || (VideoGrabber->BurstMode && (VideoGrabber->BurstCount == 0))) { // reuse the same form
            if (! SnapForm) {
               Application->CreateForm (__classid (TForm), &SnapForm);
               SnapForm->Position = poDefault;
               SnapForm->FormStyle = fsStayOnTop;
            }
            if (SnapImage) {
               SnapImage->Free();
            }
            SnapImage = new TImage (DestForm);
            DestForm = SnapForm;
            DestImage = SnapImage;
         }
         else { // create a form for each image
            Application->CreateForm (__classid (TForm), &DestForm);
            DestForm->Position = poDefault;
            DestForm->FormStyle = fsStayOnTop;
            DestImage = new TImage (DestForm);
            DestImage->Parent = DestForm;
         }

         DestForm->ClientWidth = Frame_Bitmap->Width;
         DestForm->ClientHeight = Frame_Bitmap->Height;
         DestImage->AutoSize = true;

         bool AlternateMethod = False; // you can choose one of the 2 methods below (set True to test the 2nd)

         if (! AlternateMethod) { // this code reassign the bitmap handle to the picture (saves CPU)
            DestImage->Picture->Bitmap->Handle = Frame_Bitmap->Handle;
            Frame_Bitmap->ReleaseHandle();
         }
         else { // this code makes a copy of the bitmap
            DestImage->Picture->Assign (Frame_Bitmap);
         }

         DestImage->Parent = DestForm;

         if (! DestForm->Visible) {
            DestForm->Show();
         }
      } break;
    }
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//                             PLAYER TAB
////////////////////////////////////////////////////////////////////////////////

void __fastcall TfrmMainForm::btnPlayerOpenFileClick(TObject *Sender)
{
   dlgOpen->FileName = "*.mpg;*.mpeg;*.avi;*.wav;*.mp3;*.asf;*.wmv;*.vob";
   if (dlgOpen->Execute()) {
      edtPlayerClip->Text =dlgOpen->FileName;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnGetClipDurationClick(TObject *Sender)
{
   __int64 Duration;
   __int64 FrameCount;

   if (VideoGrabber->AVIDuration (edtPlayerClip->Text, Duration, FrameCount)) {
       mmoLog->Lines->Add ("Duration: " + IntToStr (Duration));
       mmoLog->Lines->Add ("Frame count: " + IntToStr (FrameCount));
   }
   else {
       mmoLog->Lines->Add ("failed to get info for this clip");
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnOpenClipClick(TObject *Sender)
{
   VideoGrabber->PlayerFileName = edtPlayerClip->Text;
   VideoGrabber->OpenPlayer();

}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnPlayerCloseClipClick(TObject *Sender)
{
 VideoGrabber->ClosePlayer();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnOpenLastClipRecordedClick(TObject *Sender)
{
   if (VideoGrabber->Last_Recording_FileName != "") {
      VideoGrabber->PlayerFileName = VideoGrabber->Last_Recording_FileName;
      VideoGrabber->OpenPlayer();
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnOpenLastClipPlayedClick(TObject *Sender)
{
   if (VideoGrabber->Last_Clip_Played != "") {
      VideoGrabber->PlayerFileName = VideoGrabber->Last_Clip_Played;
      VideoGrabber->OpenPlayer();
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnPlayerOpenStreamingURLClick(
      TObject *Sender)
{
   edtPlayerClip->Text = "mms://www.datastead.com/demo/demo.wmv";
}
//---------------------------------------------------------------------------


void __fastcall TfrmMainForm::btnPlayerCloseFileClick(TObject *Sender)
{
   VideoGrabber->ClosePlayer();
}
//---------------------------------------------------------------------------


void __fastcall TfrmMainForm::btnPlayerPauseClick(TObject *Sender)
{
   VideoGrabber->PausePlayer();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnPlayerStopClick(TObject *Sender)
{
   VideoGrabber->StopPlayer();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnPlayerStartPlayingClick(TObject *Sender)
{
   VideoGrabber->RunPlayer();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnPlayerBackwardsClick(TObject *Sender)
{
   VideoGrabber->RunPlayerBackwards();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnPlayerRewindClick(TObject *Sender)
{
   VideoGrabber->RewindPlayer();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnPlayerFastForwardClick(TObject *Sender)
{
   VideoGrabber->FastForwardPlayer();

}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnAutoPlayClick(TObject *Sender)
{
   VideoGrabber->AutoStartPlayer = btnAutoPlay->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkPlayerAudioRenderingClick(TObject *Sender)
{
   VideoGrabber->PlayerAudioRendering = chkPlayerAudioRendering->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rdgDVPlaybackSizeClick(TObject *Sender)
{
   VideoGrabber->PlayerDVSize = TDVSize (rdgDVPlaybackSize->ItemIndex);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::chkRefreshPausedDisplayClick(TObject *Sender)
{
   VideoGrabber->PlayerRefreshPausedDisplay = chkRefreshPausedDisplay->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::updPlayerSpeedRatioClick(TObject *Sender,
      TUDBtnType Button)
{
   VideoGrabber->PlayerSpeedRatio = float (updPlayerSpeedRatio->Position) / 1000;
   edtPlayerSpeedRatio->Text = FormatFloat ("0.0", VideoGrabber->PlayerSpeedRatio);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::updPlayerFastSeekSpeedClick(TObject *Sender,
      TUDBtnType Button)
{
   VideoGrabber->PlayerFastSeekSpeedRatio = updPlayerFastSeekSpeed->Position;
   edtPlayerFastSeekSpeed->Text = IntToStr (VideoGrabber->PlayerFastSeekSpeedRatio);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberPlayerOpened(TObject *Sender)
{
   mmoLog->Lines->Add ("");
   mmoLog->Lines->Add ("playing " + VideoGrabber->PlayerFileName);
   if (VideoGrabber->IsPlayerAudioStreamAvailable == ts_True) {
      mmoLog->Lines->Add ("this video clip has an audio stream");
   }
   else if (VideoGrabber->IsPlayerAudioStreamAvailable == ts_False) {
      mmoLog->Lines->Add ("this video clip does not have audio");
   }
   mmoLog->Lines->Add ("");
   mmoLog->Lines->Add ("video codec: " + VideoGrabber->PlayerVideoCodec);
   mmoLog->Lines->Add ("audio codec: " + VideoGrabber->PlayerAudioCodec);
   mmoLog->Lines->Add ("");
   mmoLog->Lines->Add ("Duration: " + FormatFloat ("0.00", VideoGrabber->PlayerDuration / 10000000));
   mmoLog->Lines->Add ("Frame count: " + IntToStr (VideoGrabber->PlayerFrameCount));
   mmoLog->Lines->Add ("Frame rate: " + FormatFloat ("0.00", VideoGrabber->PlayerFrameRate));
   mmoLog->Lines->Add ("size:" + IntToStr (VideoGrabber->VideoWidth) + "x" + IntToStr (VideoGrabber->VideoHeight));
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberPlayerBufferingData(
      TObject *Sender, bool StartingToBuffer)
{
   if (StartingToBuffer) {
      mmoLog->Lines->Add ("buffering data...");
   }
   else {
      mmoLog->Lines->Add ("buffering completed.");
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberPlayerEndOfStream(
      TObject *Sender)
{
   mmoLog->Lines->Add ("End of stream.");
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//                           NETWORK STREAMING TAB
////////////////////////////////////////////////////////////////////////////////



void __fastcall TfrmMainForm::btnViewDirectNetworkStreamingClick(
      TObject *Sender)
{
   if (VideoGrabber->StreamingURL != "") {
      ShellExecute (NULL, "open", VideoGrabber->StreamingURL.c_str(), NULL, NULL, SW_SHOWNORMAL);
   }
}
//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::rdgNetworkStreamingClick(TObject *Sender)
{
   VideoGrabber->NetworkStreaming = TNetworkStreaming (rdgNetworkStreaming->ItemIndex);
   btnViewDirectNetworkStreaming->Enabled = VideoGrabber->NetworkStreaming == ns_ASFDirectNetworkStreaming;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtNetworkPortChange(TObject *Sender)
{
   VideoGrabber->ASFNetworkPort = StrToIntDef (edtNetworkPort->Text, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtNetworkMaxUsersChange(TObject *Sender)
{
   VideoGrabber->ASFNetworkMaxUsers = StrToIntDef (edtNetworkMaxUsers->Text, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtPublishingPointChange(TObject *Sender)
{
   VideoGrabber->ASFMediaServerPublishingPoint = edtPublishingPoint->Text;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnShowProfilesListClick(TObject *Sender)
{
   lsbProfiles->Items->Text = VideoGrabber->ASFProfiles;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtASFVideoWidthChange(TObject *Sender)
{
   VideoGrabber->ASFVideoWidth = StrToIntDef (edtASFVideoWidth->Text, -1);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtASFVideoHeightChange(TObject *Sender)
{
   VideoGrabber->ASFVideoHeight = StrToIntDef (edtASFVideoHeight->Text, -1);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtASFVideoBitRateChange(TObject *Sender)
{
   VideoGrabber->ASFVideoBitRate = StrToIntDef (edtASFVideoBitRate->Text, -1);
}
//---------------------------------------------------------------------------


void __fastcall TfrmMainForm::edtASFVideoMaxKeyFrameSpacingChange(
      TObject *Sender)
{
   VideoGrabber->ASFVideoMaxKeyFrameSpacing = StrToIntDef (edtASFVideoMaxKeyFrameSpacing->Text, -1);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtASFVideoQualityChange(TObject *Sender)
{
   VideoGrabber->ASFVideoQuality = StrToIntDef (edtASFVideoQuality->Text, -1);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtASFAudioChannelsChange(TObject *Sender)
{
   VideoGrabber->ASFAudioChannels = StrToIntDef (edtASFAudioChannels->Text, -1);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkASFFixedFrameRateClick(TObject *Sender)
{
   VideoGrabber->ASFFixedFrameRate = chkASFFixedFrameRate->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cmbASFDeinterlaceModeChange(TObject *Sender)
{
   VideoGrabber->ASFDeinterlaceMode = TASFDeinterlaceMode (cmbASFDeinterlaceMode->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtProfileIndexChange(TObject *Sender)
{
   VideoGrabber->ASFProfile = StrToIntDef (edtProfileIndex->Text, -1);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::lsbProfilesClick(TObject *Sender)
{
   if (lsbProfiles->Items->Count > 0) {
      VideoGrabber->ASFProfile = lsbProfiles->ItemIndex;
      edtProfileIndex->Text = IntToStr (VideoGrabber->ASFProfile);
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberDirectNetworkStreamingHostUrl(
      TObject *Sender, AnsiString HostUrl, AnsiString HostName,
      int HostPort)
{
   mmoLog->Lines->Add ("streaming URL: " + HostUrl);
   mmoLog->Lines->Add ("streaming host: " + HostName);
   mmoLog->Lines->Add ("streaming port: " + IntToStr (HostPort));
   btnViewDirectNetworkStreaming->Enabled = true;
}
//---------------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////////////
//                            TV TUNER TAB
////////////////////////////////////////////////////////////////////////////////


void __fastcall TfrmMainForm::rdgTunerInputTypeClick(TObject *Sender)
{
   VideoGrabber->TVTunerInputType = TTunerInputType (rdgTunerInputType->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnSetCountryCodeClick(TObject *Sender)
{
   int CountryCode;

   CountryCode = StrToIntDef (edtCountryCode->Text, -1);
   if (CountryCode >= 0) {
      VideoGrabber->TVCountryCode = CountryCode;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnSelectClick(TObject *Sender)
{
   int Channel;

   Channel  = StrToIntDef (edtTVChannel->Text, -1);
   if (Channel != -1) {
      VideoGrabber->TVChannel = Channel;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkSetFrequencyOverrideClick(TObject *Sender)
{
   int Channel;
   int Frequency;

   Channel  = StrToIntDef (edtTVChannel->Text, -1);
   Frequency = StrToIntDef (edtChannelFrequency->Text, -1);
   if ((Channel != -1) && (Frequency != -1)) {
      VideoGrabber->TVSetChannelFrequencyOverride (Channel, Frequency);
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnDeleteFrequencyOverrideClick(
      TObject *Sender)
{
   int Channel;

   Channel  = StrToIntDef (edtTVChannel->Text, -1);
   if (Channel != -1) {
      VideoGrabber->TVSetChannelFrequencyOverride (Channel, -1);
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkEnableFrequencyOverridesClick(
      TObject *Sender)
{
   VideoGrabber->TVUseFrequencyOverrides = chkEnableFrequencyOverrides->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnStartScanClick(TObject *Sender)
{
   VideoGrabber->TVStartAutoScan();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnStopScanClick(TObject *Sender)
{
   if (VideoGrabber->TVStopAutoScan()) {
      mmoChannelLog->Lines->Add ("stopping autoscan...");
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnResetAllOverridesClick(TObject *Sender)
{
   VideoGrabber->TVClearFrequencyOverrides();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::VideoGrabberTVChannelScanStarted(
      TObject *Sender, int MinChannel, int MaxChannel)
{
   AnsiString InputType;

   switch (VideoGrabber->TVTunerInputType) {
      case TunerInputCable   : InputType = "cable input";   break;
      case TunerInputAntenna : InputType = "antenna input"; break;
   }
   mmoChannelLog->Lines->Add ("scanning country code " + IntToStr (VideoGrabber->TVCountryCode) + " (" + InputType + ") from channel " + IntToStr (MinChannel) + " to " + IntToStr (MaxChannel) + "...");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberTVChannelSelected(
      TObject *Sender, int Channel, bool Locked, int DefaultVideoFrequency,
      int OverriddenVideoFrequency, int TunerVideoFrequency,
      int TunerAudioFrequency)
{
   AnsiString sFrequencyInfo;

   if (Locked) {
      sFrequencyInfo = IntToStr (Channel) + ": " + FormatFloat ("0.00", TunerVideoFrequency / 1000000) + " Mhz (" + IntToStr (TunerVideoFrequency) + " Hz";
      if (OverriddenVideoFrequency != -1) {
         sFrequencyInfo = sFrequencyInfo + ", overridden";
      }
      sFrequencyInfo = sFrequencyInfo + ")";
      mmoChannelLog->Lines->Add (sFrequencyInfo);
   }
   else {
      if (OverriddenVideoFrequency != -1) {
         mmoChannelLog->Lines->Add (IntToStr (Channel) + "... overridden and ! locked");
      }
      else {
         mmoChannelLog->Lines->Add (IntToStr (Channel) + "...");
      }
   }
   edtTVChannel->Text = IntToStr (Channel);
   if (OverriddenVideoFrequency != -1) {
      edtChannelFrequency->Text = IntToStr (OverriddenVideoFrequency);
   }
   else {
      edtChannelFrequency->Text = IntToStr (TunerVideoFrequency);
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberTVChannelScanCompleted(
      TObject *Sender)
{
   mmoChannelLog->Lines->Add ("autoscan completed.");
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//                            FRAME OVERLAY TAB
////////////////////////////////////////////////////////////////////////////////



void __fastcall TfrmMainForm::chkTextOverlayEnabledClick(TObject *Sender)
{
   VideoGrabber->TextOverlay_String = mmoTextOverlayString->Text;
   VideoGrabber->TextOverlay_Enabled = chkTextOverlayEnabled->Checked;
}
//---------------------------------------------------------------------------


void __fastcall TfrmMainForm::chkTextOverlayTransparentClick(
      TObject *Sender)
{
   VideoGrabber->TextOverlay_Transparent = chkTextOverlayTransparent->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rdgTextOverlayAlignClick(TObject *Sender)
{
   VideoGrabber->TextOverlay_Align = TTextOverlayAlign (rdgTextOverlayAlign->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::mmoTextOverlayStringChange(TObject *Sender)
{
   VideoGrabber->TextOverlay_String = mmoTextOverlayString->Text;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtTextOverlayLeftChange(TObject *Sender)
{
   VideoGrabber->TextOverlay_Left = StrToIntDef (edtTextOverlayLeft->Text, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtTextOverlayTopChange(TObject *Sender)
{
   VideoGrabber->TextOverlay_Top = StrToIntDef (edtTextOverlayTop->Text, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtTextOverlayWidthChange(TObject *Sender)
{
   VideoGrabber->TextOverlay_Right = StrToIntDef (edtTextOverlayWidth->Text, 0);
}
//---------------------------------------------------------------------------


void __fastcall TfrmMainForm::btnFreeHandEraseClick(TObject *Sender)
{
   iFreeHandMouseDrawing = 0; 
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrBitmapOverlaySizeChange(TObject *Sender)
{
   edtBitmapOverlaySize->Text = IntToStr (tbrBitmapOverlaySize->Position) + "%";
   // look at the VideoGrabberFrameOverlayUsingDC event's code
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnLoadImageOverlayFromFileClick(
      TObject *Sender)
{
   TJPEGImage *Jpg;

   dlgOpen->FileName = "*.bmp;*.jpg;*.jpeg;*.jpe";
   if (dlgOpen->Execute()) {
      if (dlgOpen->FileName.Pos ("bmp")  > 0) {
         imgBitmapOverlay->Picture->LoadFromFile (dlgOpen->FileName);
      }
      else {
         Jpg = new TJPEGImage();
         Jpg->LoadFromFile (dlgOpen->FileName);
         imgBitmapOverlay->Picture->Bitmap->Canvas->Draw (0, 0, Jpg);
         Jpg->Free();
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnBitmapOverlayResetSizeClick(
      TObject *Sender)
{
   tbrBitmapOverlaySize->Position = 100;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkShapeOverlayEnabledClick(TObject *Sender)
{
   VideoGrabber->ShapeOverlay = shpShapeOverlay;
   VideoGrabber->ShapeOverlayEnabled = chkShapeOverlayEnabled->Checked;
   edtShapeOverlayLeftChange(this);
   edtShapeOverlayTopChange(this);
   edtShapeOverlayWidthChange(this);
   edtShapeOverlayHeightChange(this);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtShapeOverlayLeftChange(TObject *Sender)
{
   shpShapeOverlay->Left = StrToIntDef (edtShapeOverlayLeft->Text, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtShapeOverlayTopChange(TObject *Sender)
{
   shpShapeOverlay->Top = StrToIntDef (edtShapeOverlayTop->Text, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtShapeOverlayWidthChange(TObject *Sender)
{
   shpShapeOverlay->Width = StrToIntDef (edtShapeOverlayWidth->Text, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtShapeOverlayHeightChange(TObject *Sender)
{
   shpShapeOverlay->Height = StrToIntDef (edtShapeOverlayHeight->Text, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::cmbBitmapOverlayTransparentColorChange(
      TObject *Sender)
{
   cmbBitmapOverlayTransparentColorItemIndex = cmbBitmapOverlayTransparentColor->ItemIndex;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::updBitmapOverlayAlphaBendClick(
      TObject *Sender, TUDBtnType Button)
{
   updBitmapOverlayAlphaBendPosition = updBitmapOverlayAlphaBend->Position;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberFrameOverlayUsingDC(
      TObject *Sender, HDC Dc, DWORD FrameNumber, Int64 FrameTime,
      int FrameId)
{
   int BitmapOverlayWidth, BitmapOverlayHeight;
   int i;
   int LeftPos;
   int TopPos;
   TCanvas *Canvas;
   bool Transparent;
   bool AlphaBlend;
   int AlphaBlendValue;
   bool UseTransparentColor;
   TColor TransparentColorValue;

//    From this event you should not perform any actions with the potential to block:
//    1. do not hold a critical section or wait on another thread,
//    2. do not call any GDI or USER32.DLL APIs that might cause a window to move,
//    3. do not invoke component properties that send messages, e.g. do not read the ItemIndex property
//       of a ListBox component. If you need to read a such property, set an intermediary variable when
//       the property changes, and then read the intermediary variable from this event.

   if (! chkBitmapOverlayEnabled) return; // just a security, if the app is closing, this control could have been destroyed when the event occurs

   if (chkBitmapOverlayEnabled->Checked) {

      LeftPos = StrToIntDef (edtBitmapOvlerlayLeft->Text, 0);
      TopPos = StrToIntDef (edtBitmapOvlerlayTop->Text, 0);

      if (tbrBitmapOverlaySize->Position == 100) { // faster if (we do ! have to resize the bitmap
         BitmapOverlayWidth = imgBitmapOverlay->Picture->Bitmap->Width;
         BitmapOverlayHeight = imgBitmapOverlay->Picture->Bitmap->Height;
      }
      else {
         BitmapOverlayWidth  = imgBitmapOverlay->Picture->Bitmap->Width * tbrBitmapOverlaySize->Position / 100;
         BitmapOverlayHeight = imgBitmapOverlay->Picture->Bitmap->Height * tbrBitmapOverlaySize->Position / 100;
      }

      Transparent = false;
      UseTransparentColor = false;
      TransparentColorValue = (TColor) 0;

      AlphaBlend = false;
      AlphaBlendValue = (TColor) 0;

      if (chkBitmapOverlayTransparent->Checked ) {
         Transparent = true;
         if (chkBitmapOverlayUseTransparentColor->Checked ) {
            UseTransparentColor = true;
            switch (cmbBitmapOverlayTransparentColorItemIndex) {
               case 0: TransparentColorValue = clYellow; break;
               case 1: TransparentColorValue = clRed; break;
               case 2: TransparentColorValue = clBlue; break;
               case 3: TransparentColorValue = clWhite; break;
            }
         }
      }

      if (chkBitmapOverlayAlphaBlend->Checked ) {
         AlphaBlend = true;
         AlphaBlendValue = updBitmapOverlayAlphaBendPosition;
      }

      VideoGrabber->DrawBitmapOverFrame (imgBitmapOverlay->Picture->Bitmap, LeftPos, TopPos, BitmapOverlayWidth, BitmapOverlayHeight, Transparent, UseTransparentColor, TransparentColorValue, AlphaBlend, AlphaBlendValue);
   }

   if (iFreeHandMouseDrawing > 0) { // see VideoGrabberMouseMove event below
      Canvas = new TCanvas;
      Canvas->Handle = Dc;
      Canvas->Pen->Color = clRed;
      for (i = 0;  i < iFreeHandMouseDrawing ; i++) {
         if (FreeHandMouseDrawing[i].Right == 0) { // Right is used as boolean to know if (it is a MoveTo or a LineTo
            Canvas->MoveTo (FreeHandMouseDrawing[i].Left, FreeHandMouseDrawing[i].Top);
            Canvas->Pixels[FreeHandMouseDrawing[i].Left][FreeHandMouseDrawing[i].Top] = clRed;
         }
         else {
            Canvas->LineTo (FreeHandMouseDrawing[i].Left, FreeHandMouseDrawing[i].Top);
         }
      }
      Canvas->Free();
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberFrameOverlayUsingDIB(
      TObject *Sender, HBITMAP FrameBitmapHandle, DWORD FrameNumber,
      Int64 FrameTime, int FrameId)
{
   int i;
   bool SkipLine;
   void *Bits;
   DIBSECTION DibSection;

//    From this event you should not perform any actions with the potential to block:
//    1. do not hold a critical section or wait on another thread,
//    2. do not call any GDI or USER32.DLL APIs that might cause a window to move,
//    3. do not invoke component properties that send messages, e.g. do not read the ItemIndex property
//       of a ListBox component. If you need to read a such property, set an intermediary variable when
//       the property changes, and then read the intermediary variable from this event.

   if (! GetObject (FrameBitmapHandle, sizeof (DIBSECTION), &DibSection)) {
      return;
   }
   if (! chkLinesOverlayEnabled) return; // just a security, if the app is closing, this control could have been destroyed when the event occurs
   if (chkLinesOverlayEnabled->Checked) {
      SkipLine = false;
      for (i = 0 ; i < DibSection.dsBm.bmHeight ; i++) {
         if (SkipLine) {
            Bits = (void*) (int (DibSection.dsBm.bmBits) + (i * DibSection.dsBm.bmWidthBytes));
            ZeroMemory (Bits, DibSection.dsBm.bmWidthBytes);
         }
         SkipLine = ! SkipLine;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberFrameOverlayUsingVIDEOHDR(
      TObject *Sender, DWORD FrameNumber, Int64 FrameTime, int FrameId,
      const TVideoHdr &VideoHdr)
{
   int BLUE = 0;
   int GREEN = 1;
   int RED = 2;
   BYTE *pBitmapBits;
   int i, j;
   int VideoWidth, VideoHeight;
   int MinVert, MaxVert;
   int MinHorz, MaxHorz;
   int Color1Off, Color2Off;
   int RGBSize;

//    From this event you should not perform any actions with the potential to block:
//    1. do not hold a critical section or wait on another thread,
//    2. do not call any GDI or USER32.DLL APIs that might cause a window to move,
//    3. do not invoke component properties that send messages, e.g. do not read the ItemIndex property
//       of a ListBox component. If you need to read a such property, set an intermediary variable when
//       the property changes, and then read the intermediary variable from this event.

   if (! grbColorFiltering) return; // just a security, if the app is closing, this control could have been destroyed when the event occurs
   switch (grbColorFiltering->ItemIndex) {
      case 1:  // RED
         Color1Off = BLUE;
         Color2Off = GREEN;
         break;
      case 2:   // GREEN
         Color1Off = RED;
         Color2Off = BLUE;
         break;
      case 3:  // BLUE
         Color1Off = RED;
         Color2Off = GREEN;
         break;
      default: return; // "off" or invalid value
   }

   switch (VideoGrabber->FrameGrabberCurrentRGBFormat) {
      case fgf_RGB24:
         RGBSize = 3;
         break;
      case fgf_RGB32:
         RGBSize = 4;
         break;
      default: return; // we do not support other RGB formats in this example
   }

   pBitmapBits = VideoHdr.lpData; // pointer to the bitmap bits

   VideoWidth = VideoGrabber->VideoWidth; // to avoid invoking the VideoGrabber.VideoWidth function in the loop below
   VideoHeight = VideoGrabber->VideoHeight; // to avoid invoking the VideoGrabber.VideoHeight function in the loop below

   // to get centered vertical bounds within the frame
   MinVert = VideoHeight >> 2; // 2 bits right-shifted == divided by 4
   MaxVert = VideoHeight - MinVert;

   // to get centered horizontal bounds within the frame
   MinHorz = VideoWidth >> 2; // 2 bits right-shifted == divided by 4
   MaxHorz = VideoWidth - MinHorz;


   for (j = MinVert ; j < MaxVert ; j++) {
      for (i = MinHorz ; i < MaxHorz ; i ++) {
        *((pBitmapBits + (j * VideoWidth + i) * RGBSize) + Color1Off) = 0;
        *((pBitmapBits + (j * VideoWidth + i) * RGBSize) + Color2Off) = 0;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberMouseDown(TObject *Sender,
      int VideoWindow, TMouseButton Button, TShiftState Shift, int X,
      int Y)
{
   if (VideoWindow != -1) { // if the video is displayed
      if (chkFreeHandEnabled->Checked) {
         FreeHandMouseDrawing[iFreeHandMouseDrawing].Left = X;
         FreeHandMouseDrawing[iFreeHandMouseDrawing].Top = Y;
         FreeHandMouseDrawing[iFreeHandMouseDrawing].Right = 0; // Right is used as boolean to know if (it is a MoveTo or a LineTo
         iFreeHandMouseDrawing ++;
         FreeHandMouseDrawingEnabled = true;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberMouseMove(TObject *Sender,
      int VideoWindow, TShiftState Shift, int X, int Y)
{
   if (VideoWindow != -1) { // if the video is displayed
      if (FreeHandMouseDrawingEnabled) { // has been enabled in the MouseDown event above
         if (iFreeHandMouseDrawing < FreeHandMouseMax) {
            FreeHandMouseDrawing[iFreeHandMouseDrawing].Left = X;
            FreeHandMouseDrawing[iFreeHandMouseDrawing].Top = Y;
            FreeHandMouseDrawing[iFreeHandMouseDrawing].Right = 1; // Right is used as boolean to know if (it is a MoveTo or a LineTo
            iFreeHandMouseDrawing ++;
         }
         if (VideoGrabber->IsPlayerPaused) {
            VideoGrabber->PlayerTimePosition = VideoGrabber->PlayerTimePosition;
         }
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberMouseUp(TObject *Sender,
      int VideoWindow, TMouseButton Button, TShiftState Shift, int X,
      int Y)
{
   FreeHandMouseDrawingEnabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberResizeVideo(TObject *Sender,
      int SourceWidth, int SourceHeight)
{
   VideoGrabber->TextOverlay_Right = SourceWidth;
   trkCroppingX->Max = VideoGrabber->Cropping_XMax;
   trkCroppingY->Max = VideoGrabber->Cropping_YMax;
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//                           VIDEO PROCESSING TAB
////////////////////////////////////////////////////////////////////////////////


void __fastcall TfrmMainForm::rdgVideoRotationClick(TObject *Sender)
{
   VideoGrabber->VideoProcessing_Rotation = TVideoRotation (rdgVideoRotation->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rdgDeinterlacingClick(TObject *Sender)
{
   VideoGrabber->VideoProcessing_Deinterlacing = TVideoDeinterlacing (rdgDeinterlacing->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrBrightnessChange(TObject *Sender)
{
   VideoGrabber->VideoProcessing_Brightness = - tbrBrightness->Position;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrConstrastChange(TObject *Sender)
{
   VideoGrabber->VideoProcessing_Contrast = - tbrConstrast->Position;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrSaturationChange(TObject *Sender)
{
   VideoGrabber->VideoProcessing_Saturation = - tbrSaturation->Position;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrHueChange(TObject *Sender)
{
   VideoGrabber->VideoProcessing_Hue = tbrHue->Position;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnResetAllClick(TObject *Sender)
{
   rdgVideoRotation->ItemIndex = 0;
   rdgDeinterlacing->ItemIndex = 0;
   tbrBrightness->Position = 0;
   tbrConstrast->Position = 0;
   tbrSaturation->Position = 0;
   tbrHue->Position = 0;
   chkGreyScale->Checked = false;
   chkInvertColors->Checked = false;
   chkFlipVertical->Checked = false;
   chkFlipHorizontal->Checked = false;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkFlipVerticalClick(TObject *Sender)
{
   VideoGrabber->VideoProcessing_FlipVertical = chkFlipVertical->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkFlipHorizontalClick(TObject *Sender)
{
   VideoGrabber->VideoProcessing_FlipHorizontal = chkFlipHorizontal->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkGreyScaleClick(TObject *Sender)
{
   VideoGrabber->VideoProcessing_GrayScale = chkGreyScale->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkInvertColorsClick(TObject *Sender)
{
   VideoGrabber->VideoProcessing_InvertColors = chkInvertColors->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::vmr9FixRangeClick(TObject *Sender)
{
   // fixes a possible problem of constrast and saturation range with some video decoders, passed to the SetVMR9ImageAdjustmentValue function below
}
//---------------------------------------------------------------------------


void __fastcall TfrmMainForm::tbrVMR9BrightnessChange(TObject *Sender)
{
   VideoGrabber->SetVMR9ImageAdjustmentValue (true, vmr9_Brightness, tbrVMR9Brightness->Position, false);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrVMR9ContrastChange(TObject *Sender)
{
   VideoGrabber->SetVMR9ImageAdjustmentValue (true, vmr9_Contrast, tbrVMR9Contrast->Position, vmr9FixRange->Checked);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrVMR9SaturationChange(TObject *Sender)
{
   VideoGrabber->SetVMR9ImageAdjustmentValue (true, vmr9_Saturation, tbrVMR9Saturation->Position, vmr9FixRange->Checked);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::tbrVMR9HueChange(TObject *Sender)
{
   VideoGrabber->SetVMR9ImageAdjustmentValue (true, vmr9_Hue, tbrVMR9Hue->Position, false);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::InitVMR9Trackbar (TVMR9ImageAdjustment VMR9ImageAdjustment, TTrackBar *TrackBar)
{
   int MinValue, MaxValue, StepSize, DefaultValue, CurrentValue;

   TrackBar->Enabled = VideoGrabber->GetVMR9ImageAdjustmentBounds (true, vmr9_Brightness, MinValue, MaxValue, StepSize, DefaultValue, CurrentValue);
   if (TrackBar->Enabled) {
      TrackBar->Min = MinValue;
      TrackBar->Max = MaxValue;
      TrackBar->Frequency = StepSize;
      TrackBar->Position = CurrentValue;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberGraphBuilt(TObject *Sender)
{
   InitVMR9Trackbar (vmr9_Brightness, tbrVMR9Brightness);
   InitVMR9Trackbar (vmr9_Contrast, tbrVMR9Contrast);
   InitVMR9Trackbar (vmr9_Saturation, tbrVMR9Saturation);
   InitVMR9Trackbar (vmr9_Hue, tbrVMR9Hue);

   tbrAudioBalance->Position = VideoGrabber->AudioBalance;
   tbrAudioVolume->Position = VideoGrabber->AudioVolume;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkCroppingEnabledClick(TObject *Sender)
{
   VideoGrabber->Cropping_Enabled = chkCroppingEnabled->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkCroppingOutboundsClick(TObject *Sender)
{
   VideoGrabber->Cropping_Outbounds = chkCroppingOutbounds->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtCroppingWidthChange(TObject *Sender)
{
   VideoGrabber->Cropping_Width = StrToIntDef (edtCroppingWidth->Text, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtCroppingHeightChange(TObject *Sender)
{
   VideoGrabber->Cropping_Height = StrToIntDef (edtCroppingHeight->Text, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::trkCroppingZoomChange(TObject *Sender)
{
   VideoGrabber->Cropping_Zoom = trkCroppingZoom->Position / 100;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnCroppingZoomResetClick(TObject *Sender)
{
   trkCroppingZoom->Position = 100;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::trkCroppingYChange(TObject *Sender)
{
   // trkCroppingX->Max and trkCroppingY->Max are set from the OnResizeVideo event
   VideoGrabber->Cropping_Y = trkCroppingY->Position;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::trkCroppingXChange(TObject *Sender)
{
   // trkCroppingX->Max and trkCroppingY->Max are set from the OnResizeVideo event
   VideoGrabber->Cropping_X = trkCroppingX->Position;
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//                                DISPLAY TAB
////////////////////////////////////////////////////////////////////////////////

void __fastcall EnableOrDisableGroupBoxControls (TGroupBox *GroupBox, bool Value)
{
   int i;

   for (i = 0 ; i < GroupBox->ControlCount ; i ++) {
      GroupBox->Controls[i]->Enabled = Value;
   }
}

void __fastcall TfrmMainForm::RefreshDisplayOptions(void)
{
   if (ComponentState.Contains(csDestroying)) return; // do nothing if the app terminates

   // let's refresh the controls according to the TVideoGrabber's states

   AssignListToComboBox (cboVideoDevices, VideoDevices(), VideoGrabber->VideoDevice);
   AssignListToComboBox (cboAudioDevices, AudioDevices(), VideoGrabber->AudioDevice);

   chk1stVideoWindowActive->Checked = VideoGrabber->Display_Active;
   chk1stEmbedded->Checked          = VideoGrabber->Display_Embedded;
   chk1stFullScreen->Checked        = VideoGrabber->Display_FullScreen;
   chk1stStayOnTop->Checked         = VideoGrabber->Display_StayOnTop;
   chk1stUseVideoPort->Checked      = VideoGrabber->Display_VideoPortEnabled;
   chk1stMouseMovesWindow->Checked  = VideoGrabber->Display_MouseMovesWindow;
   chk1stAutoSize->Checked          = VideoGrabber->Display_AutoSize;
   chk1stKeepAspectRatio->Checked   = VideoGrabber->Display_KeepAspectRatio;
   rdg1stDisplayedOn->ItemIndex     = VideoGrabber->Display_Monitor;

   EnableOrDisableGroupBoxControls (grbMainVideoWindow, VideoGrabber->Display_Active);

   if (VideoGrabber->Display_Active) {
      chk1stStayOnTop->Enabled         = (! VideoGrabber->Display_Embedded) || VideoGrabber->Display_FullScreen;
      rdg1stDisplayedOn->Enabled       = MonitorsCount() > 1;
      chk1stMouseMovesWindow->Enabled  = ! VideoGrabber->Display_Embedded;
      rdg1stDisplayedOn->Enabled       = ! VideoGrabber->Display_Embedded;
      btnEG1stNewLocation->Enabled     = ! VideoGrabber->Display_Embedded;
      btnEG1stNewLocation2->Enabled    = ! VideoGrabber->Display_Embedded;
      chk1stUseVideoPort->Enabled      = VideoGrabber->IsVideoPortAvailable;
   }

   chk2ndVideoWindowActive->Checked = VideoGrabber->DualDisplay_Active;
   chk2ndEmbedded->Checked          = VideoGrabber->DualDisplay_Embedded;
   chk2ndFullScreen->Checked        = VideoGrabber->DualDisplay_FullScreen;
   chk2ndStayOnTop->Checked         = VideoGrabber->DualDisplay_StayOnTop;
   chk2ndUseVideoPort->Checked      = VideoGrabber->DualDisplay_VideoPortEnabled;
   chk2ndMouseMovesWindow->Checked  = VideoGrabber->DualDisplay_MouseMovesWindow;
   chk2ndAutoSize->Checked          = VideoGrabber->DualDisplay_AutoSize;
   chk2ndKeepAspectRatio->Checked   = VideoGrabber->DualDisplay_KeepAspectRatio;
   rdg2ndDisplayedOn->ItemIndex     = VideoGrabber->DualDisplay_Monitor;

   EnableOrDisableGroupBoxControls (grb2ndVideoWindow, VideoGrabber->DualDisplay_Active);

   if (VideoGrabber->DualDisplay_Active) {
      chk2ndStayOnTop->Enabled         = (! VideoGrabber->DualDisplay_Embedded) || VideoGrabber->DualDisplay_FullScreen;
      rdg2ndDisplayedOn->Enabled       = MonitorsCount() > 1;
      chk2ndMouseMovesWindow->Enabled  = ! VideoGrabber->DualDisplay_Embedded;
      rdg2ndDisplayedOn->Enabled       = ! VideoGrabber->DualDisplay_Embedded;
      btnEG2ndNewLocation->Enabled     = ! VideoGrabber->DualDisplay_Embedded;
      btnEG2ndNewLocation->Enabled     = ! VideoGrabber->DualDisplay_Embedded;
      chk2ndUseVideoPort->Enabled      = VideoGrabber->IsVideoPortAvailable;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkChangeCursorClick(TObject *Sender)
{
   if (chkChangeCursor->Checked) {
      VideoGrabber->VideoCursor = crCross;
   }
   else {
      VideoGrabber->VideoCursor = crDefault;
   }
}

//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::chkBorderClick(TObject *Sender)
{
   if (chkBorder->Checked) {
      VideoGrabber->BorderStyle = bsSingle;
   }
   else {
      VideoGrabber->BorderStyle = bsNone;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkTransparencyClick(TObject *Sender)
{
   // in this sample the transparent window is created in the VideoGrabberColorKeyChange
   // event below when the color key is set
   VideoGrabber->ColorKeyEnabled = chkTransparency->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rdgVideoRendererClick(TObject *Sender)
{
   VideoGrabber->VideoRenderer = TVideoRenderer (rdgVideoRenderer->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk1stVideoWindowActiveClick(TObject *Sender)
{
   VideoGrabber->Display_Active = chk1stVideoWindowActive->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk1stEmbeddedClick(TObject *Sender)
{
   VideoGrabber->Display_Embedded = chk1stEmbedded->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk1stAutoSizeClick(TObject *Sender)
{
   VideoGrabber->Display_AutoSize = chk1stAutoSize->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk1stKeepAspectRatioClick(TObject *Sender)
{
   VideoGrabber->Display_KeepAspectRatio = chk1stKeepAspectRatio->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk1stFullScreenClick(TObject *Sender)
{
   VideoGrabber->Display_FullScreen = chk1stFullScreen->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk1stStayOnTopClick(TObject *Sender)
{
   VideoGrabber->Display_StayOnTop = chk1stStayOnTop->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk1stUseVideoPortClick(TObject *Sender)
{
   VideoGrabber->Display_VideoPortEnabled = chk1stUseVideoPort->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rdg1stDisplayedOnClick(TObject *Sender)
{
   VideoGrabber->Display_Monitor = rdg1stDisplayedOn->ItemIndex; // 0 or 1
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnEG1stNewLocationClick(TObject *Sender)
{
   VideoGrabber->Display_AutoSize = True;  // the component"s video size will be used
   VideoGrabber->Display_Embedded = false; // detach from component
   VideoGrabber->Display_SetLocation (10, 10, 320, 240); // 320 and 240 are ! used because of AutoSize above
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnEG1stNewLocation2Click(TObject *Sender)
{
   VideoGrabber->Display_AutoSize = false; // we will specify the size below
   VideoGrabber->Display_Embedded = false; // detach from component
   VideoGrabber->Display_SetLocation (20, 20, 480, 360); // new size: 480 x 360
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk1stMouseMovesWindowClick(TObject *Sender)
{
   VideoGrabber->Display_MouseMovesWindow = chk1stMouseMovesWindow->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk2ndVideoWindowActiveClick(TObject *Sender)
{
   VideoGrabber->DualDisplay_Active = chk2ndVideoWindowActive->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk2ndEmbeddedClick(TObject *Sender)
{
   VideoGrabber->DualDisplay_Embedded = chk2ndEmbedded->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk2ndFullScreenClick(TObject *Sender)
{
   VideoGrabber->DualDisplay_FullScreen = chk2ndFullScreen->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk2ndAutoSizeClick(TObject *Sender)
{
   VideoGrabber->DualDisplay_AutoSize = chk2ndAutoSize->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk2ndKeepAspectRatioClick(TObject *Sender)
{
   VideoGrabber->DualDisplay_KeepAspectRatio = chk2ndKeepAspectRatio->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk2ndStayOnTopClick(TObject *Sender)
{
   VideoGrabber->DualDisplay_StayOnTop = chk2ndStayOnTop->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk2ndUseVideoPortClick(TObject *Sender)
{
   VideoGrabber->DualDisplay_VideoPortEnabled = chk2ndUseVideoPort->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk2ndMouseMovesWindowClick(TObject *Sender)
{
   VideoGrabber->DualDisplay_MouseMovesWindow = chk2ndMouseMovesWindow->Checked;
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::rdg2ndDisplayedOnClick(TObject *Sender)
{
   VideoGrabber->DualDisplay_Monitor = rdg2ndDisplayedOn->ItemIndex; // 0 or 1
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnEG2ndNewLocationClick(TObject *Sender)
{
   VideoGrabber->DualDisplay_AutoSize = True;  // the component"s video size will be used
   VideoGrabber->DualDisplay_Embedded = false; // detach from component
   VideoGrabber->DualDisplay_SetLocation (510, 10, 320, 240); // 320 and 240 are ! used because of AutoSize above
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnEG2ndNewLocation2Click(TObject *Sender)
{
   VideoGrabber->DualDisplay_AutoSize = false; // we will specify the size below
   VideoGrabber->DualDisplay_Embedded = false; // detach from component
   VideoGrabber->DualDisplay_SetLocation (520, 30, 480, 360); // new size: 480 x 360
   RefreshDisplayOptions();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberColorKeyChange(TObject *Sender,
      TColor ColorKey)
{
   if (! TransparentForm) {
      Application->CreateForm (__classid (TForm), &TransparentForm);
      TransparentForm->Width = 300;
      TransparentForm->Height = 100;
      TransparentForm->Position = poScreenCenter;
      TransparentForm->FormStyle = fsStayOnTop;
      TransparentPanel = new TPanel (TransparentForm);
      TransparentPanel->Parent = TransparentForm;
      TransparentPanel->BevelInner = bvNone;
      TransparentPanel->BevelOuter = bvNone;
      TransparentPanel->Align = alClient;
      TransparentPanel->Caption = "transparent window";
      TransparentPanel->Font->Color = clRed;
      TransparentPanel->Font->Size = 16;
   }

   TransparentForm->Color = ColorKey;
   TransparentPanel->Color = ColorKey;
   TransparentForm->Show();
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//                            MOTION DETECTOR TAB
////////////////////////////////////////////////////////////////////////////////

void __fastcall TfrmMainForm::chkMotionDetectionEnabledClick(
      TObject *Sender)
{
   VideoGrabber->MotionDetector_Enabled = chkMotionDetectionEnabled->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkCompareRedClick(TObject *Sender)
{
   VideoGrabber->MotionDetector_CompareRed = chkCompareRed->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkCompareGreenClick(TObject *Sender)
{
   VideoGrabber->MotionDetector_CompareGreen = chkCompareGreen->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkCompareBlueClick(TObject *Sender)
{
   VideoGrabber->MotionDetector_CompareBlue = chkCompareBlue->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chk_GreyScaleClick(TObject *Sender)
{
   VideoGrabber->MotionDetector_GreyScale = chk_GreyScale->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkReduceVideoNoiseClick(TObject *Sender)
{
   VideoGrabber->MotionDetector_ReduceVideoNoise = chkReduceVideoNoise->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::edtGridChange(TObject *Sender)
{
   VideoGrabber->MotionDetector_Grid = edtGrid->Text;
   if (VideoGrabber->MotionDetector_IsGridValid) {
      shpGridValid->Brush->Color = clLime;
      lblGridValid->Caption = "valid";
   }
   else {
      shpGridValid->Brush->Color = clRed;
      lblGridValid->Caption = "invalid";
   }
   mmoSensitivityGrid->Text = VideoGrabber->MotionDetector_Get2DTextGrid();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::updGlobalGridSensitivityClick(
      TObject *Sender, TUDBtnType Button)
{
   if (Button == Comctrls::btNext) {
      VideoGrabber->MotionDetector_GloballyIncOrDecSensitivity (1);
   }
   else {
      VideoGrabber->MotionDetector_GloballyIncOrDecSensitivity (-1);
   }
   edtGrid->Text = VideoGrabber->MotionDetector_Grid;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnApplyGridSizeClick(TObject *Sender)
{
   VideoGrabber->MotionDetector_SetGridSize (StrToInt (edtGridColCount->Text), StrToInt (edtGridRowCount->Text));
   edtGrid->Text = VideoGrabber->MotionDetector_Grid;
   mmoSensitivityGrid->Text = VideoGrabber->MotionDetector_Get2DTextGrid();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnSetCellsSensitivityClick(TObject *Sender)
{
   VideoGrabber->MotionDetector_SetCellSensitivity (StrToInt (edtXCell->Text), StrToInt (edtYCell->Text), StrToInt (edtCellSensitivity->Text));
   edtGrid->Text = VideoGrabber->MotionDetector_Grid;
   mmoSensitivityGrid->Text = VideoGrabber->MotionDetector_Get2DTextGrid();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnShowGridDialogClick(TObject *Sender)
{
   if ((VideoGrabber->CurrentState != cs_Preview) && (VideoGrabber->CurrentState != cs_Recording)) {
      ShowMessage ("you should start preview or recording to see the video frame under the grid");
   }
   if (! VideoGrabber->MotionDetector_Enabled) {
      ShowMessage ("you should enable the motion detector first (MotionDetector_Enabled = true) to see the video frame under the grid");
   }
   if ((VideoGrabber->MotionDetector_GridXCount > 30)  || (VideoGrabber->MotionDetector_GridYCount > 30)) {
      ShowMessage ("the grid will be too dense to be edited through the dialog, you should use SetGridCell");
   }
   VideoGrabber->MotionDetector_ShowGridDialog();
   edtGrid->Text = VideoGrabber->MotionDetector_Grid;
   mmoSensitivityGrid->Text = VideoGrabber->MotionDetector_Get2DTextGrid();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::chkTriggeredClick(TObject *Sender)
{
   VideoGrabber->MotionDetector_Triggered = chkTriggered->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnTriggerNowClick(TObject *Sender)
{
   VideoGrabber->MotionDetector_TriggerNow();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberMotionDetected(TObject *Sender,
      double GlobalMotionRatio, int MaxMotionCellX, int MaxMotionCellY,
      Pointer FrameBitmap, DWORD FrameNumber, Int64 FrameTime, int FrameId,
      bool &CaptureFrame)
{
   int MaxMotionXLocation, MaxMotionYLocation;
   Graphics::TBitmap *Frame_Bitmap;

   Frame_Bitmap = (Graphics::TBitmap*) FrameBitmap;

   shpMotionDetected->Brush->Color = clLime;
   edtMotionRatio->Text = FormatFloat ("0.000", VideoGrabber->MotionDetector_GlobalMotionRatio);

   edtMaxMotionXY->Text = "X:" + IntToStr (MaxMotionCellX) + " ; Y:" + IntToStr (MaxMotionCellY);
   if (VideoGrabber->MotionDetector_GetCellLocation (MaxMotionCellX, MaxMotionCellY, MaxMotionXLocation, MaxMotionYLocation)) {
      edtMaxMotionCoord->Text = "x: " + IntToStr (MaxMotionXLocation) + " ; y: " + IntToStr (MaxMotionYLocation);
   }

   mmoMotionGrid->Text = VideoGrabber->MotionDetector_Get2DTextMotion();
   imgMotionDetected->Picture->Assign (Frame_Bitmap);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberMotionNotDetected(
      TObject *Sender, Pointer FrameBitmap, DWORD FrameNumber,
      Int64 FrameTime, int FrameId, bool &CaptureFrame)
{
   shpMotionDetected->Brush->Color = clGreen;
   edtMotionRatio->Text = "";
   mmoMotionGrid->Text = VideoGrabber->MotionDetector_Get2DTextMotion();
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//                         "CLIP REENCODING" TAB
////////////////////////////////////////////////////////////////////////////////

void __fastcall TfrmMainForm::btnSourceVideoClipClick(TObject *Sender)
{
   dlgOpen->FileName = "*.mpg;*.mpeg;*.avi;*.wav;*.mp3;*.asf;*.wmv;*.vob";
   if (dlgOpen->Execute()) {
      edtSourceVideoClip->Text = dlgOpen->FileName;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnDurationClick(TObject *Sender)
{
   __int64 Duration;
   __int64 FrameCount;

   if (VideoGrabber->AVIDuration (edtSourceVideoClip->Text, Duration, FrameCount)) {
      edtDuration->Text = IntToStr (Duration);
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnStartReencodingClick(TObject *Sender)
{
   VideoGrabber->Reencoding_SourceVideoClip = edtSourceVideoClip->Text;
   VideoGrabber->Reencoding_NewVideoClip = edtDestinationVideoClip->Text;

   VideoGrabber->Reencoding_WMVOutput = chkWMVOutput->Checked;
   VideoGrabber->Reencoding_IncludeVideoStream = chkIncludeVideoStream->Checked;
   VideoGrabber->Reencoding_IncludeAudioStream = chkIncludeAudioStream->Checked;
   VideoGrabber->Reencoding_UseFrameGrabber = chkUseFrameGrabber->Checked;
   VideoGrabber->Reencoding_UseVideoCompressor = chkUseCurrentVideoCompressor->Checked;
   VideoGrabber->Reencoding_UseAudioCompressor = chkUseCurrentAudioCompressor->Checked;
   VideoGrabber->Reencoding_Method = TRecordingMethod (rdgReencodingMethod->ItemIndex);

   VideoGrabber->Reencoding_StartTime  = StrToInt64Def (edtStartTime->Text, -1);
   VideoGrabber->Reencoding_StopTime   = StrToInt64Def (edtStopTime->Text, -1);

   VideoGrabber->StartReencoding();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::btnStopReencodingClick(TObject *Sender)
{
   VideoGrabber->StopReencoding();
}
//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//                                    MISC.
////////////////////////////////////////////////////////////////////////////////


void __fastcall TfrmMainForm::edtStoragePathChange(TObject *Sender)
{
   VideoGrabber->StoragePath = edtStoragePath->Text;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::RefreshVideoDeviceControls(void)
{
   if (ComponentState == (TComponentState() << csDestroying)) {
      // we do nothing if the app terminates
      return;
   }

   // refresh the controls according to the TVideoGrabber's states

   chkReduceDVFrameRate->Checked = VideoGrabber->DVReduceFrameRate;
   chkReduceDVFrameRate->Enabled = VideoGrabber->IsDigitalVideoIn;

   btnPauseRecording->Enabled = VideoGrabber->RecordingCanPause;
   btnResumeRecording->Enabled = VideoGrabber->RecordingCanPause;

   mnuSendDVCommands->Enabled = VideoGrabber->IsDVCommandAvailable;

   btnStreamDialog->Enabled = ! VideoGrabber->IsDigitalVideoIn;
   btnCameraControlDialog->Enabled = VideoGrabber->IsCameraControlAvailable;
   btnVideoQualityDialog->Enabled = VideoGrabber->IsVideoQualityAvailable;
   btnVideoControlDialog->Enabled = VideoGrabber->IsVideoControlAvailable;

   cboAnalogVideoStandard->Enabled = VideoGrabber->IsAnalogVideoDecoderAvailable;
   cboVideoInputs->Enabled = VideoGrabber->IsVideoCrossbarAvailable;

   edtFrameRate->Text = FormatFloat ("0.00", VideoGrabber->FrameRate);

   rdgTunerInputType->ItemIndex = VideoGrabber->TVTunerInputType;
   edtCountryCode->Text = IntToStr (VideoGrabber->TVCountryCode);
   edtTVChannel->Text = IntToStr (VideoGrabber->TVChannel);
   lblTVTunerAvailable->Visible = ! VideoGrabber->IsTVTunerAvailable;

   rdgCompressMode->Enabled = True;
   rdgCompressType->Enabled = True;

   if (! VideoGrabber->AudioRecording) {
      rdgCompressType->ItemIndex = 0;
      rdgCompressType->Enabled = false;
   }

   lblFrameOverlayRequiresFrameGrabber->Visible = VideoGrabber->FrameGrabber == fg_Disabled;
   lblMotionDetectorRequiresFrameGrabber->Visible = VideoGrabber->FrameGrabber == fg_Disabled;

   if (frmVideoControl) {
      frmVideoControl->RefreshControls();
   }

   chkOpenDML->Enabled = VideoGrabber->RecordingMethod == rm_AVI;
   rdgCompressMode->Enabled = VideoGrabber->RecordingMethod == rm_AVI;
   rdgCompressType->Enabled = VideoGrabber->RecordingMethod == rm_AVI;
   cboVideoCompressors->Enabled = VideoGrabber->RecordingMethod == rm_AVI;
   btnVideoCompressorSettings->Enabled = VideoGrabber->RecordingMethod == rm_AVI;
   cboAudioCompressors->Enabled = VideoGrabber->RecordingMethod == rm_AVI;
   btnAudioCompressorSettings->Enabled = VideoGrabber->RecordingMethod == rm_AVI;
   chkPreallocFile->Enabled = VideoGrabber->RecordingMethod == rm_AVI;
}




void __fastcall TfrmMainForm::SendVCRCommand (TMenuItem *Sender)
{
   // THIS IS THE "TAG" PROPERTY OF EACH MENU ITEM THAT SELECTS THE CORRESPONDING TVCRCommand
   if (VideoGrabber->SendDVCommand (TDVCommand (Sender->Tag))) {
      mmoLog->Lines->Add ("DV command sent...");
   }
   else {
      if (! (VideoGrabber->CurrentState == (TCurrentState() << cs_Preview << cs_Recording))) {
         mmoLog->Lines->Add ("DV command failed-> Preview must be running!");
      }
      else {
         mmoLog->Lines->Add ("DV command failed.");
      }
   }
}

void __fastcall TfrmMainForm::VideoGrabberDVCommandCompleted(
      TObject *Sender, int NewStateValue, AnsiString NewStateLabel)
{
   mmoLog->Lines->Add ("camcorder state: " + NewStateLabel);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberLog(TObject *Sender,
      TLogType LogType, AnsiString Severity, AnsiString InfoMsg)
{
   mmoLog->Lines->Add ("[" + Severity + "] " + InfoMsg);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberInactive(TObject *Sender)
{
   mmoLog->Lines->Add ("(done)");
}
//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::ShowFormAboveVideo (TForm *Form)
{
   if (Form == NULL) return;
   Form->Top = Top + VideoGrabber->Top - Form->Height;
   Form->Left = Left + VideoGrabber->Left - Form->Width;
   if (Form->Top < 0) {
      Form->Top = 0;
   }
   if (Form->Left < 0) {
      Form->Left = 0;
   }
   Form->Show();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::mnuAboutClick(TObject *Sender)
{
   VideoGrabber->About();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberResize(TObject *Sender)
{
   int NewWidth;
   int NewHeight;

   NewWidth = VideoGrabber->Left + VideoGrabber->Width + 30;
   NewHeight = VideoGrabber->Top + VideoGrabber->Height + 60;
   if (NewWidth < StartupWidth) {
      NewWidth = StartupWidth;
   }
   if (NewHeight < StartupHeight) {
      NewHeight = StartupHeight;
   }
   if ((NewWidth >= Width) && (NewHeight >= Height)) {
      Width = NewWidth;
      Height = NewHeight;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberDeviceLost(TObject *Sender)
{
   mmoLog->Lines->Add ("DEVICE LOST!");
   AssignListToComboBox (cboVideoDevices, VideoDevices(), VideoGrabber->VideoDevice);
   AssignListToComboBox (cboAudioDevices, AudioDevices(), VideoGrabber->AudioDevice);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberDiskFull(TObject *Sender)
{
   mmoLog->Lines->Add ("DISK FULL!");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberReinitializing(TObject *Sender)
{
   mmoLog->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMainForm::VideoGrabberDeviceArrivalOrRemoval(
      TObject *Sender, bool IsDeviceArrival, bool IsVideoDevice,
      AnsiString DeviceName, int DeviceIndex)
{
   AnsiString AudioOrVideoDevice;

   if (IsVideoDevice) {
      AudioOrVideoDevice= "video device ";
   }
   else {
      AudioOrVideoDevice= "audio device ";
   }
   if (IsDeviceArrival) {
      mmoLog->Lines->Add (AudioOrVideoDevice + "arrival:");
   }
   else {
      mmoLog->Lines->Add (AudioOrVideoDevice + "removal:");
   }
   mmoLog->Lines->Add (DeviceName);
   if (IsVideoDevice) {
      AssignListToComboBox (cboVideoDevices, VideoDevices(), VideoGrabber->VideoDevice);
   }
   else {
      AssignListToComboBox (cboAudioDevices, AudioDevices(), VideoGrabber->AudioDevice);
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberVideoFromBitmapsFormatNeeded(
      TObject *Sender, HBITMAP BitmapHandle, AnsiString BMPorJPEGFile)
{
   TSearchRec SearchFiles;
   Graphics::TBitmap *BmpFile;

   if (VideoGrabber->StoragePath.Length() > 0) { // let's add a trailing slash if required
      if (VideoGrabber->StoragePath.SubString (VideoGrabber->StoragePath.Length(), 1) == "\\") {
         SearchBmpPath = VideoGrabber->StoragePath;
      }
      else {
         SearchBmpPath = VideoGrabber->StoragePath + "\\";
      }
   }
   else {
      SearchBmpPath = ".\\";
   }
   if (FindFirst (SearchBmpPath + "*.bmp", faAnyFile, SearchFiles) == 0) {
      BmpFile = new (Graphics::TBitmap);
      BmpFile->LoadFromFile (SearchBmpPath + SearchFiles.Name);
      BitmapHandle = BmpFile->Handle;
      BmpFile->ReleaseHandle();
      BmpFile->Free();
      FindClose (SearchFiles);
   }
   else {
      mmoLog->Lines->Add ("");
      mmoLog->Lines->Add ("no bitmap files found in the StoragePath directory! Capture frames to BMP files first by using the frame grabber.");
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::VideoGrabberVideoFromBitmapsNextFrameNeeded(
      TObject *Sender, bool FirstSample, HBITMAP BitmapHandle,
      AnsiString BMPorJPEGFile, bool EndOfData)
{
   // you can pass to this event:
   // - either the path to a BMP file or a JPEG file (this is the case in this example)
   // - either a the HBITMAP handle of a bitmap

   // FirstSample is true the first time the event occurs (useful to know e.g. when a file search must be restarted from the beginning)

   bool FileAvailable;

   FileAvailable = False;
   if (FirstSample || (! SearchBmpFilesOpened))  { // let's start from the first file
      if (SearchBmpFilesOpened) {
        FindClose (SearchBmpFiles);
        SearchBmpFilesOpened = false;
      }
      if (FindFirst (SearchBmpPath + "*.bmp", faAnyFile, SearchBmpFiles) == 0) {
         FileAvailable = True;
         SearchBmpFilesOpened = True;
      }
      else {
         EndOfData = True;
      }
   }
   else {
      if (FindNext (SearchBmpFiles) == 0) {
         FileAvailable = True;
      }
      else {
         FindClose (SearchBmpFiles);
         SearchBmpFilesOpened = False;
         EndOfData = True;
      };
   };

   if (FileAvailable) {
      mmoLog->Lines->Add ("processing " + SearchBmpPath + SearchBmpFiles.Name);
      BMPorJPEGFile = SearchBmpPath + SearchBmpFiles.Name;
   };
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::FormDestroy(TObject *Sender)
{
   if (SearchBmpFilesOpened) {
      FindClose (SearchBmpFiles);
      SearchBmpFilesOpened = False;
   }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMainForm::mnuExitClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------



